using System;

namespace LinearAlgebra
{
	
	public class Node
	{
		internal int Topology;
		internal double X,Y;
		public Node(double x,double y,int top)
		{
			X=x;Y=y;Topology=top; 
		}
	}
	public class SteadyHeatConduction
	{
		static double k = 4; //Thermal conductivity
		static double h = 10; //convective heat transfer coefficient

		static int xres=10;
		static int yres =5;
		static double dx = 10e-3;
		static int noofnodes = (xres+1)*(yres+1);

		static Matrix COF = new Matrix (noofnodes);
		static Matrix CON = new Matrix (noofnodes ,1);

		static Node[,] NodeMatrix = new Node[yres+1,xres+1]; 
		internal static void MeshGeometry()
		{
			for(int i=0;i<=yres;i++)
			{
				for(int j=0;j<=xres;j++)
				{
					double x = j*10e-2/xres;
					double y = (5-i)*5e-2/yres;
					int top = (i*(xres+1))+j;
					NodeMatrix[i,j]=new Node(x,y,top);			
				}
			}
		
		}//Mesh Geometry

		internal static void setSystemofEquations()
		{
			for(int i=0;i<=yres;i++)		
			{
				for(int j=0;j<=xres;j++)
				{
					Node N = NodeMatrix[i,j];
					if((i==0)&&(j==0))
					{
						
						int top=N.Topology,topxp1=NodeMatrix[i,j+1].Topology,topym1=NodeMatrix[i+1,j].Topology; 
						COF[top,top]= -2*(1+(h*dx/k));
						COF[top,topym1]=1;COF[top,topxp1]=1;
						CON[top,0]=(-h*dx/k)*(20+40+273+273);				
					}
					else if((i==0)&&(j==10))
					{
						int top=N.Topology,topxm1=NodeMatrix[i,j-1].Topology,topym1=NodeMatrix[i+1,j].Topology; 
						COF[top,top]= -2*(1+(h*dx/k));
						COF[top,topym1]=1;COF[top,topxm1]=1;
						CON[top,0]=(-h*dx/k)*(100+40+273+273);				
					}
					else if((i==5)&&(j==0))
					{
						int top=N.Topology,topxp1=NodeMatrix[i,j+1].Topology,topyp1=NodeMatrix[i-1,j].Topology; 
						COF[top,top]= -2*(1+(h*dx/k));
						COF[top,topyp1]=1;COF[top,topxp1]=1;
						CON[top,0]=(-h*dx/k)*(70+20+273+273);				
					}
					else if((i==5)&&(j==10))
					{
						int top=N.Topology,topxm1=NodeMatrix[i,j-1].Topology,topyp1=NodeMatrix[i-1,j].Topology; 
						COF[top,top]= -2*(1+(h*dx/k));
						COF[top,topyp1]=1;COF[top,topxm1]=1;
						CON[top,0]=(-h*dx/k)*(70+100+273+273);				
					}
					else if((i==0)&&(j!=0)&&(j!=10))
					{
						int top=N.Topology,topxm1=NodeMatrix[i,j-1].Topology,topxp1=NodeMatrix[i,j+1].Topology,topym1=NodeMatrix[i+1,j].Topology; 
						COF[top,top]= -2*((h*dx/k)+2);
						COF[top,topxm1]=1;COF[top,topxp1]=1;COF[top,topym1]=2;
						CON[top,0]=(-2*h*dx/k)*(40+273);			
					}
					else if((i==5)&&(j!=0)&&(j!=10))
					{
						int top=N.Topology,topxm1=NodeMatrix[i,j-1].Topology,topxp1=NodeMatrix[i,j+1].Topology,topyp1=NodeMatrix[i-1,j].Topology; 
						COF[top,top]= -2*((h*dx/k)+2);
						COF[top,topxm1]=1;COF[top,topxp1]=1;COF[top,topyp1]=2;
						CON[top,0]=(-2*h*dx/k)*(70+273);		
					}
					else if((j==0)&&(i!=0)&&(i!=5))
					{
						int top=N.Topology,topym1=NodeMatrix[i+1,j].Topology,topyp1=NodeMatrix[i-1,j].Topology,topxp1=NodeMatrix[i,j+1].Topology; 
						COF[top,top]= -2*((h*dx/k)+2);
						COF[top,topym1]=1;COF[top,topyp1]=1;COF[top,topxp1]=2;
						CON[top,0]=(-2*h*dx/k)*(20+273);			
					}
					else if((j==10)&&(i!=0)&&(i!=5))
					{
						int top=N.Topology,topym1=NodeMatrix[i+1,j].Topology,topyp1=NodeMatrix[i-1,j].Topology,topxm1=NodeMatrix[i,j-1].Topology; 
						COF[top,top]= -2*((h*dx/k)+2);
						COF[top,topym1]=1;COF[top,topyp1]=1;COF[top,topxm1]=2;
						CON[top,0]=(-2*h*dx/k)*(100+273);			
					}
					else
					{
						int top=N.Topology,topxp1=NodeMatrix[i,j+1].Topology,topxm1=NodeMatrix[i,j-1].Topology; 
						int topyp1=NodeMatrix[i-1,j].Topology,topym1=NodeMatrix[i+1,j].Topology; 
						COF[top,top]=-4;
						COF[top,topxp1]=1;COF[top,topxm1]=1;
						COF[top,topyp1]=1;COF[top,topym1]=1;				
					}			
				}
			}
		
		}// setSystem of equations

		internal static void SolveForTemperatures()
		{
			string path = @"c:/temp.txt";string text="";IoLib.textoverwrite(path,text); 
			Matrix Temp = COF.Inverse()*CON;
			for(int i=0;i<66;i++)		
			{
				double T = Temp[i,0];
				if(((i%11)==0)&&(i!=0))
				{
					Console.WriteLine();
					IoLib.textappend(path,text);text=""; 
				}
				Console.Write(T.ToString("N2")+"  ");
				text += T.ToString("N2")+"  ";
				 
			}
		}

		internal static void SteadyConductionSolve()
		{
			MeshGeometry();setSystemofEquations();SolveForTemperatures();
		
		}



		 



		
	}//[Steady Heat Conduction]
}
